# Licensed under a 3-clause BSD style license - see LICENSE.rst
"""
HITRAN Catalog Query Tool
-------------------------

:Author: Adam Ginsburg (adam.g.ginsburg@gmail.com)
"""
from .reader import download_hitran, read_hitran_file, cache_location
